﻿var CustomFloatingObject = GcSpread.Sheets.CustomFloatingObject;
function CameraShape(name, spread, srcSheet, rangeRowStart, rangeRowEnd, rangeColStart, rangeColEnd) {
    CustomFloatingObject.call(this, name);

    var sizeAdjustment = 3;

    var innerDiv = document.createElement('div');

    innerDiv.innerHTML = "<div id='" + name + "' style='width: 100%; height: 100%; border: " + sizeAdjustment + "px solid gray;'></div>";
    this.Content(innerDiv);

    var innerSpread;
    var destSheet = spread.getActiveSheet();

    destSheet.bind(GcSpread.Sheets.Events.CustomFloatingObjectLoaded, function (sender, args) {
        if (args.customFloatingObject.name() === name) {
            loadInnerSpread();
            
            innerActiveSheet = innerSpread.getActiveSheet();

            // Set size of camera shape to size of range
            var height = 0;
            var width = 0;
            for (var r = rangeRowStart; r <= rangeRowEnd; r++) {
                height += innerActiveSheet.getRowHeight(r);
            }
            for (var c = rangeColStart; c <= rangeColEnd; c++) {
                width += innerActiveSheet.getColumnWidth(c);
            }
            args.customFloatingObject.height(height + sizeAdjustment);
            args.customFloatingObject.width(width + sizeAdjustment);
            args.customFloatingObject.allowResize(false);
            $("#" + name).height(height);
            $("#" + name).width(width);

            innerSpread.refresh();
        }
    });

    srcSheet.bind(GcSpread.Sheets.Events.CellChanged, function (sender, args) {
        // Update data and styles of cells in camera shape
        if ((args.row >= rangeRowStart || args.row <= rangeRowEnd) && (args.col >= rangeColStart || args.col <= rangeColEnd)) {
            reloadInnerSpread();
        }
    });

    srcSheet.bind(GcSpread.Sheets.Events.RowHeightChanged, function (e, info) {
        // Update row heights of cells in camera shape
        reloadInnerSpread();
        changeHeight();
    });

    srcSheet.bind(GcSpread.Sheets.Events.ColumnWidthChanged, function (e, info) {
        // Update column widths of cells in camera shape
        reloadInnerSpread();
        changeWidth();
    });

    function changeHeight() {
        // Set height of camera shape to height of range
        var height = 0;
        var innerActiveSheet = innerSpread.getActiveSheet();
        for (var r = rangeRowStart; r <= rangeRowEnd; r++) {
            height += srcSheet.getRowHeight(r);
        }
        var floatingObjects = destSheet.getFloatingObjects();
        for (var i = 0; i < floatingObjects.length; i++) {
            if (floatingObjects[i].name() === name) {
                floatingObjects[i].height(height + sizeAdjustment);
            }
        }
        $("#" + name).height(height);
        innerSpread.refresh();
    }

    function changeWidth() {
        // Set width of camera shape to width of range
        var width = 0;
        var innerActiveSheet = innerSpread.getActiveSheet();
        for (var c = rangeColStart; c <= rangeColEnd; c++) {
            width += srcSheet.getColumnWidth(c);
        }
        var floatingObjects = destSheet.getFloatingObjects();
        for (var i = 0; i < floatingObjects.length; i++) {
            if (floatingObjects[i].name() === name) {
                floatingObjects[i].width(width + sizeAdjustment);
            }
        }
        $("#" + name).width(width);
        innerSpread.refresh();
    }

    function loadInnerSpread() {
        // Create a new Spread instance and load its properties
        innerSpread = new GcSpread.Sheets.Spread(document.getElementById(name)); 
        reloadInnerSpread();
    }

    function reloadInnerSpread() {
        // Reload the Spread instance's properties
        // If the camera shape is on the same sheet as the source sheet, then find the control already on the page.
        var floatingObjects = spread.getActiveSheet().getFloatingObjects();
        for (var i = 0; i < floatingObjects.length; i++) {
            if (floatingObjects[i].name() === name) {
                innerSpread = GcSpread.Sheets.findControl(document.getElementById(name));
            }
        }
        
        innerSpread.isPaintSuspended(true);
        innerSpread.tabStripVisible(false);
        innerSpread.showHorizontalScrollbar(false);
        innerSpread.showVerticalScrollbar(false);

        var innerActiveSheet = innerSpread.getSheet(0);

        // Copy the sheet from the source Spread instance to destination Spread instance via JSON
        var sourceSheetJSON = srcSheet.toJSON();
        innerActiveSheet.fromJSON(sourceSheetJSON);
        innerActiveSheet.setName("inner_" + innerActiveSheet.getName());

        // Move the view to the selected data in the sheet
        innerActiveSheet.showCell(rangeRowStart, rangeColStart, GcSpread.Sheets.VerticalPosition.top, GcSpread.Sheets.HorizontalPosition.left);

        // Set properties of the active sheet in the inner Spread instance
        innerActiveSheet.setFrozenTrailingRowCount(innerActiveSheet.getRowCount() - rangeRowStart);
        innerActiveSheet.setColumnHeaderVisible(false);
        innerActiveSheet.setRowHeaderVisible(false);
        innerActiveSheet.setIsProtected(true);
        innerActiveSheet.protectionOption().allowSelectLockedCells = false;
        innerActiveSheet.allowCellOverflow(true);

        spread.focus(true);
        innerSpread.isPaintSuspended(false);
    }
}
CameraShape.prototype = Object.create(CustomFloatingObject.prototype);
CameraShape.prototype.constructor = CustomFloatingObject;
